const axios = require("axios");

class IncidentsManager{

	/**
 * @param {string} hostNamePrefix Service Cloud host name prefix
 * @param {string} domainName Service Cloud domain name
 * @param {string} userName Service Cloud userName that have WebService API call privilege
 * @param {string} password Service Cloud user password
 */	
	constructor(hostNamePrefix, domainName, userName,password){

		this.hostNamePrefix = hostNamePrefix;
		this.domainName = domainName;
		this.userName = userName;
		this.password = password;

	}

	/**
 * Get a list of incident IDs created by the user defined by the userEmail
 * @param {string} userEmail user email
 * @param {integer} numberOfIncidentsToFetch number of open incidents to fetch from service cloud.
 * @returns {[string]} List of incidents created by the user.
 */
	async getIncidents(userEmail, numberOfIncidentsToFetch){
		try {

			// Final list of user incidents to return
			let incidents = [];

			// Build AXIOS request object
			const request = {
				method: "GET",
				baseURL: `https://${this.hostNamePrefix}.${this.domainName}/services/rest/connect/latest`,
				url: `/incidents?q=primaryContact.ParentContact.Emails.EmailList.Address == '${userEmail}' AND statusWithType.status.id != 2`, // Filter incidents using customer email
				auth:{
					username: this.userName,
					password: this.password
				},
				headers:{
					"OSvC-CREST-Application-Context" : "calling from ODA", // Set Service Cloud required headers
					"Content-Type" : "application/json"
				},
				params: {
					orderBy: "createdTime:desc", // Order results
					limit: numberOfIncidentsToFetch
				}
			};
			
			// Get REST Call response
			const response = await axios(request);

			// If data is returned, the process it further
			if(response.data.items && response.data.items.length > 0){
				// Loop over returned data and extract incidentIds
				const incidentIds = response.data.items.map(incident => incident.id);

				// Temp job array to hold a list of promises for fetching an incident full details using incidentId
				const jobs = [];

				// Loop over all incidentsIds and create a new job
				// to fetch an incident details using its incidentId
				incidentIds.forEach(async incidentId => {
					jobs.push(this.getIncidentById(incidentId));
				});

				// Execute all jobs in parallel
				incidents = await Promise.all(jobs);
			}

			// Return final incidents list
			return incidents;
      
		} catch (error) {

			// Error Message
			let errorMessage = error.message;
			
			// If this is an 'axios' specific error, then extract its details
			if(error.isAxiosError){
				errorMessage = error.response.data.detail;
			}
			
			// Throw an error that will be handled by Custom Component
			throw new Error(errorMessage);
		}
	}

	/**
 * Get the details of an incident using its incidentId
 * @param {string} incidentId incident ID
 * @returns {object} Incident Details
 */
	async getIncidentById(incidentId){
		try {

			// Build AXIOS request object
			const request = {
				method: "GET",
				baseURL: `https://${this.hostNamePrefix}.${this.domainName}/services/rest/connect/latest`,
				url: `/incidents/${incidentId}`,
				auth:{
					username: this.userName,
					password: this.password
				},
				headers:{
					"OSvC-CREST-Application-Context" : "calling from ODA",
					"Content-Type" : "application/json"
				},
				params: {
					fields: "statusWithType,subject,id,lookupName,createdTime" // Only return these fields
				}
			};
			
			// Get REST Call response
			const response = await axios(request);

			// Extract the returned incident
			const rawIncident= response.data;

			// Build a curated version of the incident
			const incident = {
				id: rawIncident.id,
				name: rawIncident.lookupName,
				subject: rawIncident.subject,
				createdTime: new Date(rawIncident.createdTime).getTime(),
				statusId: rawIncident.statusWithType.status.id,
				status: rawIncident.statusWithType.status.lookupName,
			};

			// Return incident object
			return incident;
      
		} catch (error) {
			// Error Message
			let errorMessage = error.message;
			
			// If this is an 'axios' specific error, then extract its details
			if(error.isAxiosError){
				errorMessage = error.response.data.detail;
			}
			
			// Throw an error that will be handled by Custom Component
			throw new Error(errorMessage);
		}
	}

}

module.exports = IncidentsManager;